package de.berlios.tulklut;

/**
 * Cette classe permet de traiter du texte sous forme de chaîne de caractères
 * pour le convertir en Bulgroz, comme on peut le voir dans les albums de
 * Spirou et Fantasio.
 * 
 * @brief Convertit une chaîne de caractères en Bulgroz et inversement
 * @author Michel PETIT (pemichel@wanadoo.fr)
 * @version 1.0
 */
public class Bulgroz {
  private String inputString; //!< Stocke la chaîne de caractères entrée
  private String outputString; //!< Stocke la chaîne après conversion

  /**
   * Constructeur par défaut.
   */
  public void Tulklut(){
    this.inputString  = new String();
    this.outputString = new String();
  }

  /**
   * Constructeur alternatif permettant d'indiquer directement la chaîne de
   * caractères à traiter
   * @param inputString Chaîne de charactères
   */
  public void Tulklut(String inputString){
    if(inputString != null && inputString.equals("") != true){
      this.inputString = inputString;
      this.outputString = new String();
    } else {
      this.inputString  = new String();
      this.outputString = new String();
    }
  }
  /**
   * Donne une chaîne de caractères à convertir.
   * @param string Chaîne de caractères
   */
  public void setInputString(String string){
    if(string != null && string.equals("") != true){
      inputString = new String(string);
    }
  }

  /**
   * Retourne la chaîne de caractères initiale.
   * @return String
   */
  public String getInputString(){
    return this.inputString;
  }

  /**
   * Retourne la chaîne de caractères après conversion.
   * 
   * Si la chaîne de caractères initiale n'a pas été renseignée, une chaîne
   * vide sera retournée.
   * 
   * @return String
   */
  public String getOutputString(){
    if(inputString != null && inputString.equals("") != true){
      translate();
      return this.outputString;
    } else {
      return "";
    }
  }

  /**
   * Réalise une inversion de la chaîne de caractères en prennant en compte la
   * position des majuscules et des minuscules.
   * @return String
   * @param string Chaîne de caractères à inverser
   */
  private String invert(String string){
    int i, j;                   // index partant au début et à la fin de la chaîne
    char c;                     // caractère de transition
    int size = string.length(); // taille du tableau contenant les caractères
    char[] cs;                  // tableau qui contiendra les caractères
    cs = new char[size];        // instanciation du tableau contenant les caractères

    for(i = 0 ; i < size ; i++){
      cs[i] = string.charAt(i);
    }
    
    for(i = 0 , j = size-1 ; i < j ; i++ , j--){
      if(Character.isUpperCase(string.charAt(i)) ^ Character.isUpperCase(string.charAt(j))){
        if(Character.isUpperCase(string.charAt(i))){
          c     = Character.toLowerCase(string.charAt(i));
          cs[i] = Character.toUpperCase(string.charAt(j));
        } else {
          c     = Character.toUpperCase(string.charAt(i));
          cs[i] = Character.toLowerCase(string.charAt(j));
        }
      } else {
        c     = string.charAt(i);
        cs[i] = string.charAt(j);
      }
      cs[j] = c;
    }
    
    String str = new String(cs); // chaîne résultat, inversée
    return str;
  }
  
  /**
   * Effectue la transformation.
   * @return String
   */
  private void translate(){
    int i;
    String r = getInputString();
    String p = new String();
    String o = new String();
    for(i = 0 ; i < r.length() ; i++){
      if(Character.isLetter(r.charAt(i))){
        p = p + r.charAt(i);
        if(i == r.length()-1){
          o = o + invert(p);
        }
      } else {
        o = o + invert(p) + r.charAt(i);
        p = new String();
      }
    }

    this.outputString = o;
  }
}
